<?php
namespace Setforms2\Shared\ORM;


class BasicEntity
{


    public function persist() {
        \Setforms2\Shared\Registry::getEntityManager()->persist($this);
    }


    public function delete() {
        \Setforms2\Shared\Registry::getEntityManager()->remove($this);
    }


    public static function getObject($id) {
        return \Setforms2\Shared\Registry::getEntityManager()->find(get_called_class(), $id);
    }


    public static function getObjectsByWhere($where, $params = array(), $order_by = null) {

        $entity_name = get_called_class();
        $entity_manager = \Setforms2\Shared\Registry::getEntityManager();
        $entity_class_metadata = $entity_manager->getClassMetadata($entity_name);
        $table_name = $entity_class_metadata->getTableName();
        $sql = "select e.* from {$table_name} e where {$where}";
        if ($order_by) $sql .= " order by ".$order_by;

        $result_set_mapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entity_manager);
        $result_set_mapping->addRootEntityFromClassMetadata($entity_name, "e");

        return $entity_manager->createNativeQuery($sql, $result_set_mapping)->execute($params);
    }


    public static function getObjectByWhere($where, $params = array(), $order_by = null) {
        $objects = self::getObjectsByWhere($where, $params, $order_by);
        if (!empty($objects)) return $objects[0];
        return false;
    }

    public static function getRepository() {
        return \Setforms2\Shared\Registry::getEntityManager()->getRepository(get_called_class());
    }

    private static function processFindBy($name, $arguments) {
        $columns_string = substr($name, 6);
        $find_criteria = self::buildFindCriteria($columns_string, $arguments);
        return self::getRepository()->findBy($find_criteria);
    }


    private static function processFindOneBy($name, $arguments) {
        $columns_string = substr($name, 9);
        $find_criteria = self::buildFindCriteria($columns_string, $arguments);
        return self::getRepository()->findOneBy($find_criteria);
    }


    private static function buildFindCriteria($columns_string, array $arguments) {
        // @todo fix for cases like "Entity::findByFilialIdAndAndamento(9999, $andamento);
        $columns_array = explode("And", $columns_string);
        $find_by_criteria = array();
        foreach($columns_array as $key => $column) {
            $column = lcfirst(\Doctrine\Common\Util\Inflector::classify($column));
            $find_by_criteria[$column] = $arguments[$key];
        }

        return $find_by_criteria;
    }


    public static function __callStatic($name, $arguments) {
        if($repository = self::canRepositoryHandleQuery($name)) {
            return self::repositoryHandleQuery($repository, $name, $arguments);
        }

        if (substr($name, 0, 6) === "findBy") {
            return self::processFindBy($name, $arguments);
        }

        if (substr($name, 0, 9) === "findOneBy") {
            return self::processFindOneBy($name, $arguments);
        }
    }

    protected static function canRepositoryHandleQuery($name) {
        $repository = self::getRepository();
        if ($repository && method_exists($repository, $name))
            return $repository;
        return false;
    }

    protected static function repositoryHandleQuery($repository, $name, $arguments) {
        return call_user_func_array(array($repository, $name), $arguments);
    }

    public function preRemove() {}
    public function postRemove() {}
    public function prePersist() {}
    public function postPersist() {}
    public function preUpdate() {}
    public function postUpdate() {}
    public function postLoad() {}
    public function loadClassMetadata() {}
    public function onFlush() {}

}
